/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.syncclient.sps.jdbc;

import com.funambol.syncclient.sps.common.*;


/**
 * This class provide
 * static methods about create sql statements
 *
 *
 * $Id: DataStoreSql.java,v 1.6 2007/11/28 10:45:02 nichele Exp $
 */

public class DataStoreSql {

    //----------------------------------------------------------- Public methods

    /**
     * build sql query to find record about key
     *
     * @param dataStore
     * @param record with set key
     * @return sql query
     **/
    public static String selectQuery(DataStore dataStore, Record record) {

        String queryFields = null;
        String query = null;

        String function = null;
        String nameFieldKey = null;

        Object params = null;

        for (int i=0; i < dataStore.getRecordMetadata().getFieldMetadata().length; i++) {

            function = (String) dataStore.getRecordMetadata().getFieldMetadata()[i].getAttributes().get("function");

            if (function.equals("key")) {
                nameFieldKey = dataStore.getRecordMetadata().getFieldMetadata()[i].getName();
            }

        }

        FieldMetadata[] fmd = dataStore.getRecordMetadata().getFieldMetadata();

        for(int i = 0; i < fmd.length; i++) {

            if( queryFields == null) {
                queryFields = fmd[i].getName();
            } else {
                queryFields =  queryFields + "," + " " + fmd[i].getName();
            }

        }

        query = "SELECT" +" " + queryFields + " " + "FROM" + " " + dataStore.getDataStoreName() + " " +
                "WHERE " + nameFieldKey + " = ?";

        return query;

    }



    /**
     * build sql query to insert record
     *
     * @param dataStore
     * @param record to insert
     * @return sql query
     **/
    public static String insertQuery(DataStore dataStore, Record record) {

        String fieldValue  = null;
        String fieldsValue = null;
        String fieldTmp    = null;

        String query = null;

        String nameFieldsQuery = null;

        FieldMetadata[] fmd = dataStore.getRecordMetadata().getFieldMetadata();


        for(int i = 0; i < fmd.length; i++) {

            fieldValue = "?";

            if(fieldsValue == null) {
                fieldsValue = fieldValue;
            } else {
                fieldsValue = fieldsValue + "," + " " + fieldValue;
            }


            if(nameFieldsQuery == null) {
                nameFieldsQuery = fmd[i].getName();
            } else {
                nameFieldsQuery = nameFieldsQuery + "," + " " + fmd[i].getName();
            }

        }


        query = "INSERT INTO" + " " + dataStore.getDataStoreName() + " " + "(" + nameFieldsQuery + ")" + " " + "VALUES("+ fieldsValue +")";

        return query;

    }



    /**
     * build sql query to update record
     *
     * @param dataStore
     * @param record to update
     * @return sql query
     **/
    public static String updateQuery(DataStore dataStore, Record record) {

        String fieldValue  = null;
        String fieldsValue = null;

        String queryFields = null;
        String query = null;

        String function = null;
        String nameFieldKey = null;

        for (int i=0; i < dataStore.getRecordMetadata().getFieldMetadata().length; i++) {

            function = (String) dataStore.getRecordMetadata().getFieldMetadata()[i].getAttributes().get("function");

            if (function.equals("key")) {
                nameFieldKey = dataStore.getRecordMetadata().getFieldMetadata()[i].getName();
            }

        }

        FieldMetadata[] fmd = dataStore.getRecordMetadata().getFieldMetadata();

        for(int i = 1; i < fmd.length; i++) {


            fieldValue = fmd[i].getName() + "=?";

            if(fieldsValue == null) {
                fieldsValue = fieldValue;
            } else {
                fieldsValue = fieldsValue + "," + fieldValue;
            }

        }

        query = "UPDATE" + " " + dataStore.getDataStoreName() + " SET " + fieldsValue +
                " WHERE " + nameFieldKey + " = ?";

        return query;

    }



    /**
     * build sql query to delete record
     *
     * @param dataStore
     * @param record to delete
     * @return sql query
     **/
    public static String deleteQuery(DataStore dataStore, Record record) {

        String fieldsValue = null;

        String queryFields = null;
        String query = null;

        String function = null;
        String nameFieldKey = null;

        for (int i=0; i < dataStore.getRecordMetadata().getFieldMetadata().length; i++) {

            function = (String) dataStore.getRecordMetadata().getFieldMetadata()[i].getAttributes().get("function");

            if (function.equals("key")) {
                nameFieldKey = dataStore.getRecordMetadata().getFieldMetadata()[i].getName();
            }

        }

        query = "DELETE FROM " + dataStore.getDataStoreName() +
                " WHERE " + nameFieldKey +" = ?";

        return query;

    }



    /**
     * build sql query to find all record in datastore
     *
     * @param dataStore
     * @return sql query
     **/
    public static String allRecordsQuery(DataStore dataStore) {

        String queryFields = null;
        String query = null;

        FieldMetadata[] fmd = dataStore.getRecordMetadata().getFieldMetadata();

        for(int i = 0; i < fmd.length; i++) {
            if( queryFields == null) {
                queryFields =  fmd[i].getName();
            } else {
                queryFields =  queryFields + "," + " " + fmd[i].getName();
            }

        }

        query = "SELECT" + " " + queryFields + " " + "FROM" + " " + dataStore.getDataStoreName();

        return query;

    }



    /**
     * build sql query to find all record in datastore
     *
     * @param dataStore
     * @return sql query
     **/
    public static String RecordsByStateByDateQuery(DataStore dataStore, String nameFieldState, String nameFieldTimestamp) {

        String queryFields = null;
        String query = null;

        String where = null;

        FieldMetadata[] fmd = dataStore.getRecordMetadata().getFieldMetadata();

        for(int i = 0; i < fmd.length; i++) {
            if( queryFields == null) {
                queryFields =  fmd[i].getName();
            } else {
                queryFields =  queryFields + "," + " " + fmd[i].getName();
            }

        }


        if (nameFieldState != null && nameFieldTimestamp != null) {
            where = " " + "where " + nameFieldState + " = ? and " + nameFieldTimestamp + " > ?";
        } else if (nameFieldState != null && nameFieldTimestamp == null) {
            where = " " + "where " + nameFieldState + " = ?";
        } else if (nameFieldState == null && nameFieldTimestamp != null) {
            where = " " + "where " + nameFieldTimestamp + " > ?";
        }

        query = "SELECT" + " " + queryFields + " " + "FROM" + " " + dataStore.getDataStoreName() + where;

        return query;

    }



    /**
     * build sql query to find next key in datastore
     *
     * @param dataStore
     * @return sql query
     **/
    public static String getNextKeyQuery(DataStore dataStore, String nameFieldKey) {

        String query = null;

        query = "SELECT MAX(" + nameFieldKey + ") + 1 FROM" + " " + dataStore.getDataStoreName();

        return query;

    }

}
