/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.syncclient.sps.pocketpc;

import java.util.Hashtable;

import com.funambol.syncclient.sps.pocketpc.RapiConstants;
import com.funambol.syncclient.sps.common.DataAccessException;
import com.funambol.syncclient.sps.common.RecordMetadata;

/**
 * This class collects the native methods used to interact with WindowsMobile/
 * PocketPC devices throught the RAPI library.
 *
 *
 *
 */
public class RapiDB implements RapiConstants{


    // ------------------------------------------------------------ Private data

    private String dbName;
    private String volume;
    private long dbHandle;
    private long dbOid;
    private byte[] volOid;
    private int keyField;
    private int statusField;
    private int sortField;


    // ------------------------------------------------------------ Constructors

    /**
     * Creates a new instance of RapiDB
     *
     * @param volume the volume where the database is stored - NOT NULL
     * @param dbName the database name - NOT NULL
     */
    public RapiDB(String volume, String dbName) {
        try {
            System.load(new java.io.File("jrapi.dll").getAbsolutePath());
        } catch (RuntimeException e) {
            e.printStackTrace();
            throw e;
        }

        if (volume == null) {
            throw new NullPointerException("volume is null!");
        }
        if (dbName == null) {
            throw new NullPointerException("dbName is null!");
        }

        this.volume = volume;
        this.dbName = dbName;
    }

    /**
     * Create a database with the name given in the constructor and with the
     * structure defined by the given RecordMetaDAta object.
     *
     * @param keyField position (1 based) of the key field
     * @param statusField position (1 based) of the status field
     *
     * @return true id the database has been successfully open, false otherwise
     *
     * @throws DataAccessException in case of error
     */
    public native boolean createDB(int keyField, int statusField)
    throws DataAccessException;

    /**
     * Opens the database. Remember to close it with closeDB()
     *
     * @param sortField position (1 based) of the field used as sort order
     *
     * @return true id the database has been successfully open, false otherwise
     *
     * @throws DataAccessException in case of a low-level failure
     */
    public native boolean openDB(int keyField, int statusField, int sortField)
    throws DataAccessException;

    /**
     * Closes a database previously opened.
     *
     * @throws DataAccessException in case of a low-level failure
     */
    public native void closeDB()
    throws DataAccessException;

    /**
     * Deletes the database with name specified by dbName
     *
     * throws DataAccessException in case of low-level failure
     */
    public native void deleteDB()
    throws DataAccessException;

    /**
     * Returns a record by its key
     *
     * @param key the record key
     *
     * @return the record fields as an Object[] or null if the record is not
     *         found
     *
     * @throws DataAccessException in case of database access or low-level failure
     */
    public native Object[] readRecord(String key)
    throws DataAccessException;

    /**
     * Writes the given record fields. If the record already exists, an update
     * is performed, otherwise an insert is performed
     *
     * @param key the record key
     * @param fields record fields
     *
     * @throws DataAccessException in case of database access or low-level failure
     */
    public native Object[] storeRecord(Object key, Object[] fields)
    throws DataAccessException;

    /**
     * Sets the value of a specified field of a specified record.
     *
     * @param key record key
     * @param pos field 1-based field position
     * @param value the field value
     *
     * @throws DataAccessException in case of database access or low-level failure
     */
    public native void setRecordField(Object key, int pos, Object value)
    throws DataAccessException;

    /**
     * Deletes the given record
     *
     * @param key the key of record to be removed
     *
     * @throws DataAccessException in case of error
     */
    public native void deleteRecord(Object key)
    throws DataAccessException;

    /**
     * Returns all records of the opened database. The
     *
     * @return a Object[][] containing all records of the database, where each
     *         record is represented by an array of Objects. Each record field
     *         value is encapsulated in an appropriate Java type. A zero-length
     *         array is returned if the database contains no records.
     *
     * @throws DataAccessException in case of database access or low-level failure
     */
    public native Object[][] getAllRecords()
    throws DataAccessException;

    /**
     * Returns the next record in the database with the given state.
     * The database must be opened with the modification status field as sort
     * order.
     *
     * @param state the status to be used as searching key
     *
     * @throws DataAccessException in case of low-level error
     */
    public native Object[] getNextRecord(char state)
    throws DataAccessException;
}
